/*
* Copyright (c) 2021, Intel Corporation
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
*/
//!
//! \file     vp_kernel_config.cpp
//! \brief    vp kernel config
//! \details  vp kernel config
//!

#include <utility>
#include "vp_kernel_config.h"
#include "vp_utils.h"

using namespace vp;

VpKernelConfig::VpKernelConfig()
{
}

VpKernelConfig::~VpKernelConfig()
{
}

MOS_STATUS VpKernelConfig::GetKernelParam(VpKernelID kernelId, RENDERHAL_KERNEL_PARAM &param)
{
    VP_FUNC_CALL();

    auto it = m_kernelParams.find(kernelId);
    if (it == m_kernelParams.end())
    {
        VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
    }

    // CURBE_Length in param here is 32 aligned with 5 bits right shift.
    param = (*it).second;

    return MOS_STATUS_SUCCESS;
}