/*
 * Copyright (c) 2026, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * 'Software'), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

////////////////////////////////////////////////////////////////////////////////
// !!! WARNING - AUTO GENERATED FILE. DO NOT EDIT DIRECTLY. !!!
// Generated by KernelBinToSource.exe tool
////////////////////////////////////////////////////////////////////////////////

#ifndef __IGVPHDRRENDER_ENUM_H__
#define __IGVPHDRRENDER_ENUM_H__

enum ARGS_HDRRENDER_HDR_MANDATORY
{
    HDRRENDER_HDR_MANDATORY_PRIVATE_BASE_STATELESS,
    HDRRENDER_HDR_MANDATORY_START_X,
    HDRRENDER_HDR_MANDATORY_START_Y,
    HDRRENDER_HDR_MANDATORY_DELTA_X,
    HDRRENDER_HDR_MANDATORY_DELTA_Y,
    HDRRENDER_HDR_MANDATORY_TOP_LEFT,
    HDRRENDER_HDR_MANDATORY_BOTTOM_RIGHT,
    HDRRENDER_HDR_MANDATORY_LAYER_INFO,
    HDRRENDER_HDR_MANDATORY_CONST_ALPHA,
    HDRRENDER_HDR_MANDATORY_OP_TYPE,
    HDRRENDER_HDR_MANDATORY_FILL_COLOR,
    HDRRENDER_HDR_MANDATORY_DST_WIDTH,
    HDRRENDER_HDR_MANDATORY_DST_HEIGHT,
    HDRRENDER_HDR_MANDATORY_LAYER_NUM,
    HDRRENDER_HDR_MANDATORY_DST_INFO,
    HDRRENDER_HDR_MANDATORY_SAMPLE0,
    HDRRENDER_HDR_MANDATORY_SAMPLE1,
    HDRRENDER_HDR_MANDATORY_SURF0,
    HDRRENDER_HDR_MANDATORY_SURF1,
    HDRRENDER_HDR_MANDATORY_SURF2,
    HDRRENDER_HDR_MANDATORY_SURF3,
    HDRRENDER_HDR_MANDATORY_SURF4,
    HDRRENDER_HDR_MANDATORY_SURF5,
    HDRRENDER_HDR_MANDATORY_SURF6,
    HDRRENDER_HDR_MANDATORY_SURF7,
    HDRRENDER_HDR_MANDATORY_SURF8,
    HDRRENDER_HDR_MANDATORY_SURF9,
    HDRRENDER_HDR_MANDATORY_SURF10,
    HDRRENDER_HDR_MANDATORY_SURF11,
    HDRRENDER_HDR_MANDATORY_SURF12,
    HDRRENDER_HDR_MANDATORY_SURF13,
    HDRRENDER_HDR_MANDATORY_SURF14,
    HDRRENDER_HDR_MANDATORY_SURF15,
    HDRRENDER_HDR_MANDATORY_SURF16,
    HDRRENDER_HDR_MANDATORY_SURF17,
    HDRRENDER_HDR_MANDATORY_SURF18,
    HDRRENDER_HDR_MANDATORY_SURF19,
    HDRRENDER_HDR_MANDATORY_SURF20,
    HDRRENDER_HDR_MANDATORY_SURF21,
    HDRRENDER_HDR_MANDATORY_SURF22,
    HDRRENDER_HDR_MANDATORY_SURF23,
    HDRRENDER_HDR_MANDATORY_SURF24,
    HDRRENDER_HDR_MANDATORY_SURF25,
    HDRRENDER_HDR_MANDATORY_SURF26,
    HDRRENDER_HDR_MANDATORY_SURF27,
    HDRRENDER_HDR_MANDATORY_SURF28,
    HDRRENDER_HDR_MANDATORY_SURF29,
    HDRRENDER_HDR_MANDATORY_SURF30,
    HDRRENDER_HDR_MANDATORY_SURF31,
    HDRRENDER_HDR_MANDATORY_SURF32,
    HDRRENDER_HDR_MANDATORY_SURF33,
    HDRRENDER_HDR_MANDATORY_SURF34,
    HDRRENDER_HDR_MANDATORY_SURF35,
    HDRRENDER_HDR_MANDATORY_SURF36,
    HDRRENDER_HDR_MANDATORY_SURF37,
    HDRRENDER_HDR_MANDATORY_SURF38,
    HDRRENDER_HDR_MANDATORY_SURF39,
    HDRRENDER_HDR_MANDATORY_DST_SURFACE,
    HDRRENDER_HDR_MANDATORY_DST_SURFACE_UV,
    HDRRENDER_HDR_MANDATORY_DST_SURFACE_V,
    HDRRENDER_HDR_MANDATORY_COEF_SURFACE,
    HDRRENDER_HDR_MANDATORY_INDIRECT_DATA_POINTER,
    HDRRENDER_HDR_MANDATORY_SCRATCH_POINTER,
    HDRRENDER_HDR_MANDATORY_MAX
};

#endif // __IGVPHDRRENDER_ENUM_H__
